// -*- tab-width:4 -*-

#include "ALWndProcHook.hpp"
#include "ALGuiDragDrop.hpp"
#include "ALSysCharSet.hpp"

using namespace al::system;

namespace al {
namespace gui {

/**
   CALDragDropクラスの実装に用いられるデータ。
 **/

class CALDragDropImpl : public CALWndProcHook
{
    friend class CALDragDrop;

protected:
    explicit CALDragDropImpl(CALDragDrop* pbody) : _pbody(pbody) {}

    virtual bool MyWndProc(
        LRESULT&	lresult, WSCbase* win,
        HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

private:
    /** 最後にDrag&Dropされたファイル名のリストを保持する。 */
    std::vector<std::string>	_dropped_files;

    /** このインスタンスを生成したCALDragDropクラスのインスタンス */
    CALDragDrop*	_pbody;

};

bool CALDragDropImpl::MyWndProc(
        LRESULT&	lresult, WSCbase*,
        HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    if( message != WM_DROPFILES )
        return false;

    _dropped_files.clear();

    HDROP	hdrop = (HDROP)wParam;
    size_t     n_files = DragQueryFileW(hdrop,~0u,NULL,0);
    for( size_t i_file = 0; i_file < n_files; ++i_file ) {
        std::wstring	buff;
        size_t	buffsize = DragQueryFileW(hdrop,i_file,NULL,0) + 1;
        buff.resize(buffsize);
        DragQueryFileW(hdrop,i_file,&buff[0],buffsize);
        std::string	utf8_path;
        
        ::al::CSysCharSet::GetDefSysCharSet().ConvertFromSysWCS(buff.c_str(),utf8_path);
        _dropped_files.push_back(utf8_path);
    }
        
    DragFinish(hdrop);

    _pbody->DragDropCB();

    lresult = 0;
    return true;
}


CALDragDrop::CALDragDrop()
try
{
    _pimpl = new CALDragDropImpl(this);
}
catch (...) {
    _pimpl = NULL;
    throw;
}

CALDragDrop::~CALDragDrop()
{
    if( _pimpl != NULL ) {
        delete _pimpl;
        _pimpl = NULL;
    }
}

bool CALDragDrop::InitDragDrop(WSCbase* win)
{
    if( _pimpl == NULL )
        return false;

    HWND hwnd = _pimpl->RegisterHook(win);
    if( hwnd == NULL )
        return false;

    DragAcceptFiles(hwnd,TRUE);
    return true;
}

bool CALDragDrop::GetDroppedFiles( std::vector<std::string>& dropped_files )
{
    if( _pimpl == NULL )
        return false;
    dropped_files = _pimpl->_dropped_files;
    return true;
}

void CALDragDrop::ClearDroppedFiles()
{
    if( _pimpl == NULL )
        return;
    _pimpl->_dropped_files.clear();
}

void CALDragDrop::DragDropCB()
{
}

} /* namespace gui */
} /* namespace al */
