// -*- tab-width:4 -*-
#pragma once
#if !defined(ALUTDRAGDROP_HPP_)
#define ALUTDRAGDROP_HPP_

/**@file
   @brief ドラッグ＆ドロップ管理クラスの宣言。
**/

class WSCbase;

#include <vector>
#include <string>

namespace al {
namespace gui {

/**
   ドラッグ＆ドロップを実装するための基底クラス。
**/

/** 実装用データ */
class CALDragDropImpl;

class CALDragDrop {

    friend class CALDragDropImpl;

public:
    CALDragDrop();

    virtual ~CALDragDrop();

    /** win に対して Drag&Dropが機能するように準備する。 */
    bool InitDragDrop(WSCbase* win);

    /** 最後の Drag&Drop で引き取られたファイル名のリストを取得する。
        @return Drag&Drop されていない時は false。*/
    bool GetDroppedFiles( std::vector<std::string>& dropped_files );

    /** 最後の Drag&Drop で引き取られたファイル名のリストをクリアする。 */
    void ClearDroppedFiles();

private:
    /** Drag&Drop が行なわれたタイミングで呼び出される仮想関数。 */
    virtual void DragDropCB();

private:
    /** 管理データへのポインタ。 */
    CALDragDropImpl*	_pimpl;

};

} /* namespace gui */
} /* namespace al */



#endif /*ALUTDRAGDROP_HPP_*/
